# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Core management.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: HALManager.py 28 2007-10-16 13:11:50Z andy $"

__all__ = ['HALManager']

import dbus
import dbus.service
import gobject

dbus_version = getattr(dbus, 'version', (0, 0, 0))
if dbus_version >= (0, 41, 0):
    import dbus.glib
if dbus_version >= (0, 80, 0):
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)

# Start gobject and dbus.glib threads
gobject.threads_init()
dbus.glib.init_threads()

class HALManager:
    def __init__(self):
        self.bus = dbus.SystemBus()
        self.hal_manager_obj = self.bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
        self.hal_manager = dbus.Interface(self.hal_manager_obj, 'org.freedesktop.Hal.Manager')

    def get_dev_obj(self, udi):
        return self.bus.get_object('org.freedesktop.Hal', udi)

    def get_dev_if(self, dev_obj, interface='org.freedesktop.Hal.Device'):
        return dbus.Interface(dev_obj, interface)




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Implement simple logging module.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: Log.py 33 2007-10-19 16:11:36Z andy $"

__all__ = ['logger']

import sys
import logging

# Debug levels
DEBUG_TABLE = {
    1:      [9, "DEBUG[1]"],
    2:      [8, "DEBUG[2]"],
    5:      [7, "DEBUG[5]"],
    10:     [6, "DEBUG[10]"],
    20:     [5, "DEBUG[20]"],
    50:     [4, "DEBUG[50]"],
    100:    [3, "DEBUG[100]"],
    200:    [2, "DEBUG[200]"],
}

def add_levels(table):
    """ Add custom logging levels """
    for level in table.keys():
        logging.addLevelName(table[level][0], table[level][1])

add_levels(DEBUG_TABLE)

def convert_level(level, table):
    """ Convert logging levels using lookup table """
    # Look up level in the table.
    try:
        return table[level][0]
    except KeyError:
        # Search closest level
        keys = table.keys()
        keys.sort()
        for key in keys:
            if level < key:
                return table[key][0]
        return table[keys[-1]][0]

class Log:
    """ Simply logger """
    def __init__(self):
        formatter = logging.Formatter("%(asctime)s %(levelname)s %(message)s",
                                      "%b %d %H:%M:%S")
        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(formatter)
        self.logger = logging.getLogger("miniHalLib")
        self.logger.addHandler(handler)

    def set_logger(self, new_logger):
        """ Update logger object """
        self.logger = new_logger

    def set_log_level(self, level):
        """ Setup level of logging """
        self.logger.setLevel(level)

    def set_debug_level(self, level):
        """ Setup debug level of logging """
        if level == 0:
            new_level = logging.DEBUG
        else:
            new_level = convert_level(level, DEBUG_TABLE)
        self.set_log_level(new_level)

    def log(self, level, msg):
        """ Print message at given level """
        self.logger.log(level, msg)

    def debug(self, level, msg):
        """ Print debug message """
        if level == 0:
            new_level = logging.DEBUG
        else:
            new_level = convert_level(level, DEBUG_TABLE)
        self.log(new_level, msg)

    def info(self, msg):
        """ Print info message """
        self.log(logging.INFO, msg)

    def warning(self, msg):
        """ Print warning message """
        self.log(logging.WARNING, msg)

    def error(self, msg):
        """ Print error message """
        self.log(logging.ERROR, msg)

logger = Log()




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Device with its properties as the dict interface.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: HALDevice.py 36 2007-11-11 12:18:30Z andy $"

__all__ = ['HALDevice', 'Device']

import dbus

from minihallib.Log import logger

class HALDevice:
    """ Special class to store device and its dbus connection """
    def __init__(self, dev_obj, property_modified_cb=None):
        self.__dev_obj = dev_obj
        self.property_modified_cb = property_modified_cb
        self.__dev_if = dbus.Interface(self.__dev_obj, 'org.freedesktop.Hal.Device')
        self.__properties = self.__dev_if.GetAllProperties()
        self.__parent = None

    def dbg(self, level, msg):
        logger.debug(level, "%s: %s" % (self.__class__.__name__, msg))

    def keys(self):
        return self.__properties.keys()

    def has_key(self, key):
        return self.__properties.has_key(key)

    def __delitem__(self, key):
        if self.__properties.has_key(key):
            del self.__properties[key]

    def __setitem__(self, key, xproperty):
        self.__properties[key] = xproperty

    def __getitem__(self, key):
        return self.__properties[key]

    def get(self, key, result=None):
        return self.__properties.get(key, result)

    def udi(self):
        return self.get('info.udi')

    def interface(self, dbus_interface='org.freedesktop.Hal.Device'):
        return dbus.Interface(self.__dev_obj, dbus_interface)

    def update_property(self, xproperty):
        """ Called when single property has been modified """
        (name, added, removed) = xproperty
        self.dbg(100, "Property is modified: '%s'" % name)

        try:
            modified = self.__dev_if.PropertyExists(name)
            if modified:
                self.__properties[name] = self.__dev_if.GetProperty(name)
                self.dbg(50, "Property '%s' has been updated, new value: '%s'" % (name, self.__properties[name]))
        except dbus.DBusException, e:
            logger.warning("%s" % str(e))
            modified = False

        if not modified:
            if self.__properties.has_key(name):
                del self.__properties[name]

        if self.property_modified_cb is not None:
            self.property_modified_cb(self, name, modified)

    def property_modified(self, changes, properties_list):
        """ Property modified event handler """
        self.dbg(50, "%d properties are modified" % changes)
        for xproperty in properties_list:
            self.update_property(xproperty)

    def set_parent(self, parent=None):
        self.__parent = parent

    def get_parent(self):
        return self.__parent

class Device:
    """ Represent HALDevice in more abstract and readonly way """
    def __init__(self, hal_device):
        self.hal_device = hal_device

    def dbg(self, level, msg):
        logger.debug(level, "%s: %s" % (self.__class__.__name__, msg))

    def get(self, key, result=None):
        return self.hal_device.get(key, result)

    def udi(self):
        return self.hal_device.udi()




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
lspci, lsusb and similar tools implementation based on HAL.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: ListBus.py 33 2007-10-19 16:11:36Z andy $"

__all__ = ['ListBus']

from minihallib.HALManager import HALManager
from minihallib.HALDevice import HALDevice

class ListBus(HALManager):
    """ Emulate system's lspci, lsusb amd similar tools """
    def __init__(self):
        HALManager.__init__(self)
        self.__devices = {}

    def clear(self, bus=None):
        if bus is None:
            self.__devices = {}
        elif self.__devices.has_key(bus):
            del self.__devices[bus]
        else:
            return False
        return True

    def fill(self, bus):
        if bus is None:
            return False

        self.__devices[bus] = {}

        for udi in self.hal_manager.GetAllDevices():
            dev_obj = self.get_dev_obj(udi)
            dev_if = self.get_dev_if(dev_obj)
            if dev_if.PropertyExists('info.bus') and dev_if.GetProperty('info.bus') == bus:
                self.__devices[bus][udi] = HALDevice(dev_obj, None)

        return True

    def refresh(self, bus):
        if bus is None:
            return False

        self.clear(bus)
        return self.fill(bus)

    def get(self, udi, result=None):
        for bus in self.__devices.keys():
            if self.__devices[bus].has_key(udi):
                return self.__devices[bus][udi]
        return result

    def lsbus(self):
        return self.__devices.keys()

    def lsdev(self, bus=None, cls=None):
        """ Return tuples of connected devices by their bus and class """
        if bus is None:
            buses = self.__devices.keys()
        else:
            buses = [bus]

        result = []
        for bus in buses:
            for udi in self.__devices[bus].keys():
                device = self.__devices[bus][udi]
                dev_cls = (device.get('%s.device_class' % bus, 0) << 8) + device.get('%s.device_subclass' % bus, 0)
                if cls is None or cls == dev_cls:
                    result.append(udi)
        return result




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
DBus messages handler.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: __init__.py 16 2007-10-02 14:35:57Z andy $"

__all__ = ['HALDevice', 'HALEventerThread', 'HALHandler', 'HALManager', 'HALPlugin', 'HALWrapper', 'ListBus', 'Log']




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Capability based plugin implementation for HAL handler.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: HALPlugin.py 16 2007-10-02 14:35:57Z andy $"

__all__ = ['HALPlugin']

import dbus

from minihallib.HALDevice import HALDevice

def udi_add_signal_receiver(bus, udi, name, call_back):
    bus.add_signal_receiver(call_back, name, 'org.freedesktop.Hal.Device', 'org.freedesktop.Hal', udi)

def udi_remove_signal_receiver(bus, udi, name, call_back):
    bus.remove_signal_receiver(call_back, name, 'org.freedesktop.Hal.Device', 'org.freedesktop.Hal', udi)

def make_hal_device(bus, udi, property_modified_cb=None):
    """ Make HALDevice object with 'PropertyModified' callback """
    dev_obj = bus.get_object('org.freedesktop.Hal', udi)
    hal_device = HALDevice(dev_obj, property_modified_cb)
    parent_udi = hal_device.get('info.parent')
    try:
        parent_obj = bus.get_object('org.freedesktop.Hal', parent_udi)
    except TypeError:
        parent_obj = None
    hal_device.set_parent(parent_obj)
    udi_add_signal_receiver(bus, udi, 'PropertyModified', hal_device.property_modified)
    return hal_device

class HALPlugin:
    """ Common HALEventer plugin """
    def __init__(self, bus, hal_manager, capability, property_modified_cb=None):
        self.__bus = bus
        self.__hal_manager = hal_manager
        self.__capability = capability
        self.property_modified_cb = property_modified_cb
        self.__devices = {}

    def add(self, udi):
        """ Add single device by its udi """
        dev_obj = self.__bus.get_object('org.freedesktop.Hal', udi)
        dev_if = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
        if dev_if.QueryCapability(self.__capability):
            self.__devices[udi] = make_hal_device(self.__bus, udi, self.property_modified_cb)
            return True
        return False

    def remove(self, udi, remove_cb=None):
        """ Remove single device by its udi """
        if self.__devices.has_key(udi):
            udi_remove_signal_receiver(self.__bus, udi, 'PropertyModified', self.__devices[udi].property_modified)
            if remove_cb is not None:
                remove_cb(self.__capability, udi)
            del self.__devices[udi]
            return True
        return False

    def fill(self):
        """ Fill devices list by current connected devices """
        for udi in self.__hal_manager.FindDeviceByCapability(self.__capability):
            self.__devices[udi] = make_hal_device(self.__bus, udi, self.property_modified_cb)

    def clear(self):
        """ Clear devices list """
        self.__devices.clear()

    def keys(self):
        """ Return list of connected devices """
        return self.__devices.keys()

    def get(self, udi, result=None):
        """ Get device by its udi """
        return self.__devices.get(udi, result)




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Basic thread to wait for HAL events.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: HALEventerThread.py 34 2007-11-11 11:53:05Z andy $"

__all__ = ['HALEventerThread']

import threading
import time
import gobject

from minihallib.Log import logger
from minihallib.HALManager import HALManager
from minihallib.HALPlugin import HALPlugin

class HALEventerThread(threading.Thread, HALManager):
    """ Thread to wait for HAL events """
    def __init__(self):
        threading.Thread.__init__(self)
        HALManager.__init__(self)

        self.hal_manager.connect_to_signal('DeviceAdded', self.device_added_callback)
        self.hal_manager.connect_to_signal('DeviceRemoved', self.device_removed_callback)

        self.plugins = {}
        self.loop = gobject.MainLoop()

    def dbg(self, level, msg):
        logger.debug(level, "%s: %s" % (self.__class__.__name__, msg))

    def register_plugin(self, capability, property_modified_cb=None, fill=True):
        """ Register plugin for given capability """
        if self.plugins.has_key(capability):
            logger.error("Plugin for capability '%s' is already registered" % capability)
            return False

        self.plugins[capability] = HALPlugin(self.bus, self.hal_manager, capability, property_modified_cb)
        self.dbg(20, "Plugin '%s' has been registered" % capability)
        if fill:
            self.plugins[capability].fill()
        return True

    def unregister_plugin(self, capability, clear=True):
        """ Unregister plugin for given capability """
        if self.plugins.has_key(capability):
            if clear:
                self.plugins[capability].clear()
            self.dbg(20, "Unregistering plugin '%s'..." % capability)
            del self.plugins[capability]
            return True

        logger.error("Plugin for capability '%s' isn't registered" % capability)
        return False

    def device_init(self, capability):
        """ Virtual function will be called with connected devices at start """
        pass

    def run(self):
        """ Thread loop """
        self.dbg(20, "Waiting for media...")
        for capability in self.plugins.keys():
            self.device_init(capability)
        self.loop.run()

    def quit(self):
        self.dbg(20, "Exiting...")
        self.loop.quit()

    def device_add(self, capability, udi):
        """ Virtual function will be called when device is added """
        pass

    def device_remove(self, capability, udi):
        """ Virtual function will be called when device is removed """
        pass

    def device_added_callback(self, udi):
        """ DBus message handler """
        self.dbg(50, "Adding device: %s..." % udi)
        for capability in self.plugins.keys():
            if self.plugins[capability].add(udi):
                self.device_add(capability, udi)

    def device_removed_callback(self, udi):
        """ DBus message handler """
        self.dbg(50, "Removing %s..." % udi)
        for capability in self.plugins.keys():
            self.plugins[capability].remove(udi, self.device_remove)

def test(time_circle):
    """ Test suite """
    eventer = HALEventerThread()

    #eventer.register_plugin('volume')
    eventer.register_plugin('net')
    eventer.unregister_plugin('foo bar')
    eventer.register_plugin('storage')

    try:
        eventer.start()
        for xstep in xrange(time_circle):
            print "%d..." % (xstep+1)

            if xstep == time_circle/2:
                eventer.register_plugin('volume')
                #eventer.register_plugin('net')
                eventer.register_plugin('storage')
                eventer.unregister_plugin('storage')

                print "Registered plugins:", eventer.plugins.keys()

            time.sleep(1)
        eventer.quit()
    except KeyboardInterrupt:
        eventer.quit()




# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Thread to handle HAL events.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: HALHandler.py 22 2007-10-06 17:06:49Z andy $"

__all__ = ['HALHandler']

import Queue

from minihallib.HALEventerThread import HALEventerThread

class HALHandlerThread(HALEventerThread):
    """ Thread to handle HAL events """
    def __init__(self, handler):
        HALEventerThread.__init__(self)
        self.handler = handler

    def device_init(self, capability):
        for udi in self.plugins[capability].keys():
            self.handler.push_message("device init", capability, self.plugins[capability].get(udi))

    def device_add(self, capability, udi):
        self.handler.push_message("device added", capability, self.plugins[capability].get(udi))

    def device_remove(self, capability, udi):
        self.handler.push_message("device removed", capability, self.plugins[capability].get(udi))

class HALHandler:
    """ HAL events wrapper """
    def __init__(self):
        self.queue = Queue.Queue()
        self.thread = HALHandlerThread(self)
        self.thread.setDaemon(True)

    def register_plugin(self, capability, property_modified=None, fill=True):
        self.thread.register_plugin(capability, property_modified, fill)

    def unregister_plugin(self, capability, clear=True):
        self.thread.unregister_plugin(capability, clear)

    def start(self):
        self.thread.start()

    def quit(self):
        self.thread.quit()
        self.thread.join()

    def push_message(self, msg, capability, data):
        """ Puts an item into message queue.
        msg is a string (for example, 'device added').
        capability is a string name of registered capability (for example, 'volume').
        data is a dictionary. """
        self.queue.put((msg, capability, data))

    def poll_message(self):
        """ Takes one message from queue and returns (msg, capability, data) tuple or
        None if there is no messages. """
        try:
            return self.queue.get(False)
        except Queue.Empty:
            return None




#!/usr/bin/python -tt
# -*- coding: UTF-8 -*-
# vim: sw=4 ts=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: setup.py 36 2007-11-11 12:18:30Z andy $"

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = "0.1.10"

setup(name = "minihallib",
    description = "Library to handle HAL devices and events",
    version = version,
    author = "Andy Shevchenko",
    author_email = "andy@smile.org.ua",
    url = "http://www.smile.org.ua/trac/minihallib",
    license = "GPLv2+ or AFL",
    packages = ['minihallib'],
    long_description = "Python threaded library to handle HAL devices and their events",
    keywords = "python dbus hal",
    platforms = "Python 2.4 and later",
    classifiers = [
        "Development Status :: 4 - Beta",
        "Operating System :: Unix",
        "License :: OSI Approved :: Academic Free License (AFL)",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Hardware",
        ]
    )



# -*- coding: UTF-8 -*-
# vim: ts=4 sw=4 et:
#
# Copyright (C) 2007 Andy Shevchenko
#
# Licensed under the Academic Free License version 2.1
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
Wrapper around HAL devices and their events.
"""

__author__ = "Andy Shevchenko <andy@smile.org.ua>"
__revision__ = "$Id: HALWrapper.py 24 2007-10-16 06:00:18Z andy $"

__all__ = ['HALWrapper']

import threading
import Queue
import time

from minihallib.Log import logger
from minihallib.HALHandler import HALHandler

# Keep CPU available
IDLE_TIMEOUT    = 0.05

class MsgReceiver(threading.Thread):
    """ Basic class to handle received HAL events """
    def __init__(self, receiver, sleep_time=IDLE_TIMEOUT):
        threading.Thread.__init__(self)
        self.__receiver = receiver
        self.sleep_time = sleep_time
        self.queue = Queue.Queue()
        self.__exit = False

    def dbg(self, level, msg):
        logger.debug(level, "%s: %s" % (self.__class__.__name__, msg))

    def push(self, msg, data):
        self.queue.put((msg, data))

    def run(self):
        """ Thread loop """
        self.dbg(100, "Running...")
        while not self.__exit:
            if self.queue.empty():
                time.sleep(self.sleep_time)
                continue

            result = self.queue.get(False)
            self.__receiver(result)

    def quit(self):
        self.__exit = True

class MsgSender(threading.Thread):
    """ Thread to asynchronously send HAL events """
    def __init__(self, hal_handler, sleep_time=IDLE_TIMEOUT):
        threading.Thread.__init__(self)
        self.__hal_handler = hal_handler
        self.sleep_time = sleep_time
        self.__receivers = {}
        self.__exit = False

    def dbg(self, level, msg):
        logger.debug(level, "%s: %s" % (self.__class__.__name__, msg))

    def register_receiver(self, capability, receiver):
        self.dbg(20, "Register receiver for '%s'" % capability)
        self.__receivers[capability] = MsgReceiver(receiver)
        self.__receivers[capability].start()

    def unregister_receiver(self, capability):
        if self.__receivers.has_key(capability):
            self.dbg(20, "Unregister receiver for '%s'" % capability)
            self.__receivers[capability].quit()
            del self.__receivers[capability]

    def run(self):
        """ Thread loop """
        while not self.__exit:
            result = self.__hal_handler.poll_message()

            if result is None:
                time.sleep(self.sleep_time)
                continue

            msg, capability, data = result
            if self.__receivers.has_key(capability):
                self.__receivers[capability].push(msg, data)

    def quit(self):
        self.__exit = True

class HALWrapper:
    """ Wrapper for HAL handler and message sender """
    def __init__(self):
        self.hal_wrapped_objs = {}
        self.handler = HALHandler()
        self.msgsender = MsgSender(self.handler)

    def dbg(self, level, msg):
        logger.debug(level, "%s: %s" % (self.__class__.__name__, msg))

    def get(self, capability, result=None):
        """ Return object with given capability if it was registered """
        return self.hal_wrapped_objs.get(capability, result)

    def clear_events(self):
        while self.handler.poll_message() is not None:
            pass

    def register(self, hwc):
        """ Register 'PropertyModified' callback and message receiver """
        if self.hal_wrapped_objs.has_key(hwc.capability):
            logger.error("'%s' already registered" % hwc.capability)
            return False

        self.dbg(100, "Register '%s'" % hwc.capability)

        self.hal_wrapped_objs[hwc.capability] = hwc

        self.handler.register_plugin(hwc.capability, hwc.property_modified_cb)
        self.msgsender.register_receiver(hwc.capability, hwc.receiver)

        return True

    def unregister(self, hwc):
        """ Unregister 'PropertyModified' callback and message receiver """
        if self.hal_wrapped_objs.has_key(hwc.capability):
            hwc.quit()

            self.handler.unregister_plugin(hwc.capability)
            self.msgsender.unregister_receiver(hwc.capability)

            self.dbg(100, "Unregister '%s'" % hwc.capability)
            return True

        logger.error("'%s' isn't registered" % hwc.capability)
        return False

    def start(self):
        """ Run HAL eventer interface """
        self.handler.start()
        self.msgsender.start()

    def stop(self):
        """ Stop HAL eventer interface """
        for capability in self.hal_wrapped_objs.keys():
            self.unregister(self.hal_wrapped_objs[capability])

        self.msgsender.quit()
        self.handler.quit()

class TestCls:
    def __init__(self, capability=None):
        self.capability = capability
        self.property_modified_cb = self.property_modified_cb_f
        self.receiver = self.receiver_f

    def quit(self):
        print 'Quiting... (%s)' % self.capability

    def property_modified_cb_f(self, hal_device, name, modified):
        print self.capability, name, modified

    def receiver_f(self, pkt):
        print self.capability, pkt

def test(time_circle):
    wrapper = HALWrapper()
    wrapper.register(TestCls('volume'))
    wrapper.register(TestCls('net'))
    try:
        wrapper.start()
        for xstep in xrange(time_circle):
            print "%d..." % (xstep+1)
            time.sleep(1)
        wrapper.stop()
    except KeyboardInterrupt:
        wrapper.stop()

